@@ -144,32 +144,47 @@ static inline int32_t sum(int32_t *ind, int32_t length){
   return s;
 }
 
+/*
+  - len1 is the current dimension of the quotient
+  - len_lm is the number of polynomials in the GB
+  - basis will contain the final monomial basis
+  - new_basis
+*/
 static inline int32_t generate_new_elts_basis(int32_t nvars, int32_t *ind,
-                                              long len1, long len_lm,
-                                              int32_t *basis1, int32_t *new_basis,
+                                              len_t len1, 
+                                              int32_t *basis, int32_t *new_basis,
+                                              len_t len_lm,
                                               int32_t *bexp_lm){
-  long c = 0;
-  for(int32_t n = nvars-1; n >= 0; n--){
-    for(int32_t i = ind[nvars-1-n]; i < len1; i++){
-      for(int32_t k = 0; k < nvars; k++){
-        new_basis[c*nvars+k] = basis1[i*nvars+k];
+
+  len_t c = 0;
+  len_t def = 0;
+  for(int64_t n = nvars-1; n >= 0; n--){
+    for(len_t i = ind[nvars-1-n]; i < len1; i++){
+      for(int64_t k = 0; k < nvars; k++){
+        new_basis[c*nvars+k] = basis[i*nvars+k];
       }
       new_basis[c*nvars+n]++;
       if(!is_divisible_lexp(nvars, len_lm, (new_basis+c*nvars), bexp_lm)){
         c++;
       }
+      else{
+        def++;
+      }
     }
   }
+
   return c;
 }
 
-
+/* takes as input an exponent and an index and returns the degree of the
+   variable next to the index */
 static inline int degree_prev_var(int32_t *exp, long n){
   return(exp[n+1]);
 }
 
 static inline void update_indices(int32_t *ind, int32_t *basis,
                                   long dquot, long new_length, long nvars){
+
   ind[0] = dquot;
   for(long n = nvars - 2; n >= 0 ; n--){
     for(long i = ind[nvars-1-n-1]; i < dquot + new_length; i++){
@@ -198,7 +213,6 @@ static inline int32_t *monomial_basis(long length, long nvars,
   (*dquot) = 0;
 
   if(is_divisible_lexp(nvars, length, (basis), bexp_lm)){
-    fprintf(stderr, "Stop ideal is generated by 1\n");
     free(basis);
     return NULL;
   }
@@ -212,8 +226,9 @@ static inline int32_t *monomial_basis(long length, long nvars,
 #endif
 
   int32_t *new_basis = malloc(sizeof(int32_t) * nvars * (sum(ind, nvars) + nvars));
-  int32_t new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                            basis, new_basis, bexp_lm);
+  int32_t new_length = generate_new_elts_basis(nvars, ind, (*dquot),
+                                               basis, new_basis,
+                                               length, bexp_lm);
 #ifdef DEBUGHILBERT
   //  display_monomials_from_array(stderr, new_length, new_basis, gens);
   fprintf(stderr, "%ld new elements.\n", new_length);
@@ -250,8 +265,9 @@ static inline int32_t *monomial_basis(long length, long nvars,
       exit(1);
     }
     new_basis=new_basis2;
-    new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                         basis, new_basis, bexp_lm);
+    new_length = generate_new_elts_basis(nvars, ind, (*dquot),
+                                         basis, new_basis,
+                                         length, bexp_lm);
 #ifdef DEBUGHILBERT
     fprintf(stderr, "%ld new elements.\n", new_length);
 #endif
@@ -291,8 +307,9 @@ static inline int32_t *monomial_basis_colon(long length, long nvars,
 
 
   int32_t *new_basis = malloc(sizeof(int32_t) * nvars * (sum(ind, nvars) + nvars)); 
-  long new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                            basis, new_basis, bexp_lm);
+  long new_length = generate_new_elts_basis(nvars, ind, (*dquot),
+                                            basis, new_basis,
+                                            length, bexp_lm);
 #ifdef DEBUGHILBERT
   display_monomials_from_array(stderr, new_length, new_basis, gens);
   fprintf(stderr, "%ld new elements.\n", new_length);
@@ -329,8 +346,9 @@ static inline int32_t *monomial_basis_colon(long length, long nvars,
       exit(1);
     }
     new_basis=new_basis2;
-    new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                         basis, new_basis, bexp_lm);
+    new_length = generate_new_elts_basis(nvars, ind, (*dquot),
+                                         basis, new_basis,
+                                         length, bexp_lm);
 #ifdef DEBUGHILBERT
     fprintf(stderr, "%ld new elements.\n", new_length);
 #endif
@@ -369,8 +387,9 @@ static inline int32_t *monomial_basis_colon_no_zero(long length, long nvars,
 #endif
 
   int32_t *new_basis = malloc(sizeof(int32_t) * nvars * (sum(ind, nvars) + nvars)); 
-  long new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                            basis, new_basis, bexp_lm);
+  long new_length = generate_new_elts_basis(nvars, ind, (*dquot),
+                                            basis, new_basis,
+                                            length, bexp_lm);
 #ifdef DEBUGHILBERT
   display_monomials_from_array(stderr, new_length, new_basis, gens);
   fprintf(stderr, "%ld new elements.\n", new_length);
@@ -407,8 +426,9 @@ static inline int32_t *monomial_basis_colon_no_zero(long length, long nvars,
       exit(1);
     }
     new_basis=new_basis2;
-    new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                         basis, new_basis, bexp_lm);
+    new_length = generate_new_elts_basis(nvars, ind, (*dquot),
+                                         basis, new_basis,
+                                         length, bexp_lm);
 #ifdef DEBUGHILBERT
     fprintf(stderr, "%ld new elements.\n", new_length);
 #endif
@@ -841,7 +861,7 @@ static inline void
 copy_extrapoly_in_vector(uint32_t* vector, long ncols, int32_t *lmb,
 			 len_t pos, const bs_t * const tbr,
 			 const ht_t * const bht, int32_t* evi,
-			 const stat_t *st, const int nv, const long maxdeg){
+			 const md_t *st, const int nv, const long maxdeg){
   len_t idx = tbr->lmps[pos];
   /* printf ("idx=%d\n",idx); */
   /* if (tbr->hm[idx] == NULL) {*/
@@ -881,7 +901,7 @@ static inline void
 copy_extrapoly_in_vector_no_zero(uint32_t* vector, long ncols,
 				 int32_t *lmb, len_t pos, const bs_t * const tbr,
 				 const ht_t * const bht, int32_t* evi,
-				 const stat_t *st, const int nv,
+				 const md_t *st, const int nv,
 				 const long maxdeg){
 
   len_t idx = tbr->lmps[pos];
@@ -914,7 +934,7 @@ static inline void
 copy_extrapoly_in_matrixcol(sp_matfglmcol_t* matrix, long nrows,
 			    int32_t *lmb, len_t pos, const bs_t * const tbr,
 			    const ht_t * const bht, int32_t* evi,
-			    const stat_t *st, const int nv,
+			    const md_t *st, const int nv,
 			    const long maxdeg){
   len_t idx = tbr->lmps[pos];
   /* printf ("idx=%d\n",idx); */
@@ -956,7 +976,7 @@ static inline void
 copy_extrapoly_in_matrixcol_no_zero(sp_matfglmcol_t* matrix, long nrows,
 				    int32_t *lmb, len_t pos, const bs_t * const tbr,
 				    const ht_t * const bht, int32_t* evi,
-				    const stat_t *st, const int nv,
+				    const md_t *st, const int nv,
 				    const long maxdeg){
 
   len_t idx = tbr->lmps[pos];
@@ -1196,23 +1216,26 @@ static inline int32_t *monomial_basis_enlarged(long length, long nvars,
 
   int32_t deg = 0;
 
-  if(is_divisible_lexp(nvars, length, (basis), bexp_lm)){
-    fprintf(stderr, "Stop ideal is generated by 1\n");
+  if(is_divisible_lexp(nvars, length, basis, bexp_lm)){
     free(basis);
     return NULL;
   }
   else{
     (*dquot)++;
   }
+
   int32_t *ind = calloc(nvars, sizeof(int32_t));
 
 #ifdef DEBUGHILBERT
   fprintf(stderr, "new = %ld \n", sum(ind, nvars) + nvars);
 #endif
+  uint64_t len_newbs = nvars * (sum(ind, nvars) + nvars);
+
+  int32_t *new_basis = malloc(sizeof(int32_t) * len_newbs);
+  /* generates monomial basis candidates of degree 1 */
+  int32_t new_length = generate_new_elts_basis(nvars, ind, (*dquot), 
+                                               basis, new_basis, length, bexp_lm);
 
-  int32_t *new_basis = malloc(sizeof(int32_t) * (nvars) * (sum(ind, nvars) + nvars));
-  int32_t new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                               basis, new_basis, bexp_lm);
   deg++;
 #ifdef DEBUGHILBERT
   //  display_monomials_from_array(stderr, new_length, new_basis, gens);
@@ -1220,14 +1243,16 @@ static inline int32_t *monomial_basis_enlarged(long length, long nvars,
 #endif
 
   while(new_length>0 && deg <= maxdeg){
+    uint64_t len_bs = ((*dquot) + new_length) * (nvars);
     int32_t *basis2 = realloc(basis,
-                              ((*dquot) + new_length) * (nvars) * sizeof(int32_t *));
+                              ((*dquot) + new_length) * (nvars) * sizeof(int32_t));
     if(basis2==NULL){
       fprintf(stderr, "Issue with realloc\n");
       exit(1);
     }
 
     basis = basis2;
+
     for(long i = 0; i < new_length; i++){
       for(long k = 0; k < nvars; k++){
         (basis)[((*dquot) + i)*(nvars)+k] = (new_basis)[i*(nvars)+k];
@@ -1235,6 +1260,8 @@ static inline int32_t *monomial_basis_enlarged(long length, long nvars,
     }
 
     update_indices(ind, basis, *dquot, new_length, nvars);
+
+
     (*dquot) += new_length;
 
 #ifdef DEBUGHILBERT
@@ -1244,16 +1271,21 @@ static inline int32_t *monomial_basis_enlarged(long length, long nvars,
     fprintf(stderr, "new = %ld \n", sum(ind, nvars) + nvars);
 #endif
 
+    len_newbs = nvars * (sum(ind, nvars) + nvars);
+
     int32_t *new_basis2 = realloc(new_basis,
-                                  sizeof(int32_t) * (nvars) * (sum(ind, nvars) + nvars));
+                                  sizeof(int32_t) * len_newbs);
     if(new_basis==NULL){
       fprintf(stderr, "Issue with realloc\n");
       exit(1);
     }
     new_basis=new_basis2;
-    new_length = generate_new_elts_basis(nvars, ind, (*dquot), length,
-                                         basis, new_basis, bexp_lm);
+
+    new_length = generate_new_elts_basis(nvars, ind, (*dquot),
+                                         basis, new_basis,
+                                         length, bexp_lm);
     deg++;
+
 #ifdef DEBUGHILBERT
     fprintf(stderr, "%ld new elements.\n", new_length);
 #endif
@@ -1478,7 +1510,7 @@ static inline sp_matfglm_t * build_matrixn(int32_t *lmb, long dquot, int32_t bld
 static inline sp_matfglmcol_t *
 build_matrixn_colon(int32_t *lmb, long dquot, int32_t bld,
 		    int32_t **blen, int32_t **bexp, int32_t *bcf, int32_t *bexp_lm,
-		    bs_t *tbr, ht_t *bht, stat_t *st,
+		    bs_t *tbr, ht_t *bht, md_t *st,
 		    const exp_t * const mul, bs_t * bs,
 		    const int nv, const long fc, const long maxdeg,
 		    const data_gens_ff_t *gens,
@@ -1633,9 +1665,9 @@ build_matrixn_colon(int32_t *lmb, long dquot, int32_t bld,
     }
   }
   tbr = initialize_basis(st);
+  tbr->ht = bht;
 #if REDUCTION_ALLINONE
-  import_input_data_nf_ff_32(tbr, bht, st, 0, tobereduced,
-			     lens, exps, (void *)cfs);
+  import_input_data(tbr, st, 0, tobereduced, lens, exps, (void *)cfs, NULL);
   tbr->ld = tbr->lml  =  tobereduced;
   /* printf ("%ld imported\n",tobereduced); */
   for (int k = 0; k < tobereduced; ++k) {
@@ -1644,8 +1676,9 @@ build_matrixn_colon(int32_t *lmb, long dquot, int32_t bld,
   /* printf ("polynomials to be reduced\n"); */
   /* print_msolve_polynomials_ff(stdout, 0, tbr->lml, tbr, bht, */
   /* 			      st, gens->vnames, 0); */
-  int success = core_nf(&tbr, &bht, &st, mul, bs);
-  if (!success) {
+  int32_t err = 0;
+  tbr = core_nf(tbr, st, mul, bs, &err);
+  if (err) {
     printf("Problem with normalform, stopped computation.\n");
     exit(1);
   }
@@ -1817,8 +1850,7 @@ build_matrixn_colon(int32_t *lmb, long dquot, int32_t bld,
 				      tbr, bht, evi, st, nv, maxdeg);
 #else /* reduction one by one */
 	  int success;
-	  import_input_data_nf_ff_32(tbr, bht, st, count_nf, count_nf+1,
-				     lens, exps, (void *)cfs);
+	  import_input_data(tbr, st, count_nf, count_nf+1, lens, exps, (void *)cfs, NULL);
 	  tbr->ld = tbr->lml  =  1;
 	  /* printf ("1 imported\n"); */
 	  for (int k = 0; k < 1; ++k) {
@@ -1867,8 +1899,8 @@ build_matrixn_colon(int32_t *lmb, long dquot, int32_t bld,
   }
 #else /* the shifts of phi are now reduced */
   tbr = initialize_basis(st);
-  import_input_data_nf_ff_32(tbr, bht, st, count_not_lm, tobereduced,
-			     lens, exps, (void *)cfs);
+  tbr->ht = bht;
+  import_input_data(tbr, st, count_not_lm, tobereduced, lens, exps, (void *)cfs, NULL);
   tbr->ld = tbr->lml  =  2*nv-2;
   /* printf ("%d imported\n",2*nv-2); */
   for (int k = 0; k < 2*nv-2; ++k) {
@@ -1918,7 +1950,7 @@ static inline sp_matfglmcol_t *
 build_matrixn_colon_no_zero(int32_t *lmb, long dquot, int32_t bld,
 			    int32_t **blen, int32_t **bexp,
 			    int32_t *bcf, int32_t *bexp_lm, bs_t *tbr,
-			    ht_t *bht, stat_t *st,
+			    ht_t *bht, md_t *st,
 			    const exp_t * const mul, bs_t * bs,
 			    const int nv, const long fc, const long maxdeg,
 			    const data_gens_ff_t *gens,
@@ -2082,8 +2114,8 @@ build_matrixn_colon_no_zero(int32_t *lmb, long dquot, int32_t bld,
     }
   }
   tbr = initialize_basis(st);
-  import_input_data_nf_ff_32(tbr, bht, st, 0, tobereduced,
-			     lens, exps, (void *)cfs);
+  tbr->ht = bht;
+  import_input_data(tbr, st, 0, tobereduced, lens, exps, (void *)cfs, NULL);
   tbr->ld = tbr->lml  =  tobereduced;
   /* printf ("%ld imported\n",tobereduced); */
   for (int k = 0; k < tobereduced; ++k) {
@@ -2092,8 +2124,9 @@ build_matrixn_colon_no_zero(int32_t *lmb, long dquot, int32_t bld,
   /* printf ("polynomials to be reduced\n"); */
   /* print_msolve_polynomials_ff(stdout, 0, tbr->lml, tbr, bht, */
   /* 			      st, gens->vnames, 0); */
-  int success = core_nf(&tbr, &bht, &st, mul, bs);
-  if (!success) {
+  int32_t err = 0;
+  tbr = core_nf(tbr, st, mul, bs, &err);
+  if (err) {
     printf("Problem with normalform, stopped computation.\n");
     exit(1);
   }
@@ -3071,7 +3104,7 @@ static inline sp_matfglm_t * build_matrixn_from_bs_trace(int32_t **bdiv_xn,
 
 #undef REDUCTION_ALLINONE
 
-static inline int32_t *get_lm_from_bs(bs_t *bs, const ht_t *ht){
+static inline int32_t *get_lm_from_bs(const bs_t * const bs, const ht_t * const ht){
   hm_t *dt;
   const len_t nelts = bs->lml;
   const int nv    = ht->nv;
