@@ -39,17 +39,16 @@ void reset_hash_table_indices(
         );
 
 ht_t *initialize_basis_hash_table(
-    stat_t *st
+    md_t *st
     );
 
 ht_t *copy_hash_table(
-    const ht_t *bht,
-    const stat_t *st
+    const ht_t *bht
     );
 
 ht_t *initialize_secondary_hash_table(
-    const ht_t *bht,
-    const stat_t *st
+    const ht_t * const ht,
+    const md_t * const md
     );
 
 void free_shared_hash_data(
