@@ -164,14 +164,12 @@ static hm_t *reduce_dense_row_by_known_pivots_sparse_ff_8(
         const uint32_t mul= (uint32_t)(fc - dr[i]);
         dts   = pivs[i];
         if (i < ncl) {
-            cfs   = bs->cf_8[dts[COEFFS]];
             /* set corresponding bit of reducer in reducer bit array */
             if (tr > 0) {
                 rba[i/32] |= 1U << (i % 32);
             }
-        } else {
-            cfs   = mcf[dts[COEFFS]];
         }
+        cfs   = mcf[dts[COEFFS]];
 #if defined HAVE_AVX512_F
         const uint16_t mul8 = (uint16_t)(fc - dr[i]);
         mulv  = _mm512_set1_epi16(mul8);
@@ -882,6 +880,12 @@ static void probabilistic_sparse_reduced_echelon_form_ff_8(
     /* we fill in all known lead terms in pivs */
     hm_t **pivs   = (hm_t **)calloc((unsigned long)ncols, sizeof(hm_t *));
     memcpy(pivs, mat->rr, (unsigned long)mat->nru * sizeof(hm_t *));
+    j = nrl;
+    for (i = 0; i < mat->nru; ++i) {
+        mat->cf_8[j]      = bs->cf_8[mat->rr[i][COEFFS]];
+        mat->rr[i][COEFFS] = j;
+        ++j;
+    }
 
     /* unkown pivot rows we have to reduce with the known pivots first */
     hm_t **upivs  = mat->tr;
@@ -1059,6 +1063,8 @@ static int exact_application_sparse_reduced_echelon_form_ff_8(
     const len_t ncr   = mat->ncr;
     const len_t ncl   = mat->ncl;
 
+    const int32_t nthrds = st->in_final_reduction_step == 1 ? 1 : st->nthrds;
+
     /* we fill in all known lead terms in pivs */
     hm_t **pivs   = (hm_t **)calloc((unsigned long)ncols, sizeof(hm_t *));
     memcpy(pivs, mat->rr, (unsigned long)mat->nru * sizeof(hm_t *));
@@ -1067,11 +1073,11 @@ static int exact_application_sparse_reduced_echelon_form_ff_8(
     hm_t **upivs  = mat->tr;
 
     int64_t *dr  = (int64_t *)malloc(
-            (unsigned long)(st->nthrds * ncols) * sizeof(int64_t));
+            (unsigned long)(nthrds * ncols) * sizeof(int64_t));
     /* mo need to have any sharing dependencies on parallel computation,
      * no data to be synchronized at this step of the linear algebra */
     int flag  = 1;
-#pragma omp parallel for num_threads(st->nthrds) \
+#pragma omp parallel for num_threads(nthrds) \
     private(i, j, k, sc) \
     schedule(dynamic)
     for (i = 0; i < nrl; ++i) {
@@ -1193,6 +1199,8 @@ static void exact_trace_sparse_reduced_echelon_form_ff_8(
     const len_t ncr   = mat->ncr;
     const len_t ncl   = mat->ncl;
 
+    const int32_t nthrds = st->in_final_reduction_step == 1 ? 1 : st->nthrds;
+
     /* we fill in all known lead terms in pivs */
     hm_t **pivs   = (hm_t **)calloc((unsigned long)ncols, sizeof(hm_t *));
     memcpy(pivs, mat->rr, (unsigned long)mat->nru * sizeof(hm_t *));
@@ -1201,10 +1209,10 @@ static void exact_trace_sparse_reduced_echelon_form_ff_8(
     hm_t **upivs  = mat->tr;
 
     int64_t *dr  = (int64_t *)malloc(
-            (unsigned long)(st->nthrds * ncols) * sizeof(int64_t));
+            (unsigned long)(nthrds * ncols) * sizeof(int64_t));
     /* mo need to have any sharing dependencies on parallel computation,
      * no data to be synchronized at this step of the linear algebra */
-#pragma omp parallel for num_threads(st->nthrds) \
+#pragma omp parallel for num_threads(nthrds) \
     private(i, j, k, sc) \
     schedule(dynamic)
     for (i = 0; i < nrl; ++i) {
@@ -1322,20 +1330,34 @@ static void exact_sparse_reduced_echelon_form_ff_8(
     const len_t ncr   = mat->ncr;
     const len_t ncl   = mat->ncl;
 
+    const int32_t nthrds = st->in_final_reduction_step == 1 ? 1 : st->nthrds;
+
     len_t bad_prime = 0;
 
     /* we fill in all known lead terms in pivs */
     hm_t **pivs   = (hm_t **)calloc((unsigned long)ncols, sizeof(hm_t *));
-    memcpy(pivs, mat->rr, (unsigned long)mat->nru * sizeof(hm_t *));
+    if (st->in_final_reduction_step == 0) {
+        memcpy(pivs, mat->rr, (unsigned long)mat->nru * sizeof(hm_t *));
+    } else {
+        for (i = 0;  i < mat->nru; ++i) {
+            pivs[mat->rr[i][OFFSET]] = mat->rr[i];
+        }
+    }
+    j = nrl;
+    for (i = 0; i < mat->nru; ++i) {
+        mat->cf_8[j]      = bs->cf_8[mat->rr[i][COEFFS]];
+        mat->rr[i][COEFFS] = j;
+        ++j;
+    }
 
     /* unkown pivot rows we have to reduce with the known pivots first */
     hm_t **upivs  = mat->tr;
 
     int64_t *dr  = (int64_t *)malloc(
-            (unsigned long)(st->nthrds * ncols) * sizeof(int64_t));
+            (unsigned long)(nthrds * ncols) * sizeof(int64_t));
     /* mo need to have any sharing dependencies on parallel computation,
      * no data to be synchronized at this step of the linear algebra */
-#pragma omp parallel for num_threads(st->nthrds) \
+#pragma omp parallel for num_threads(nthrds) \
     private(i, j, k, sc) \
     schedule(dynamic)
     for (i = 0; i < nrl; ++i) {
@@ -1411,7 +1433,7 @@ static void exact_sparse_reduced_echelon_form_ff_8(
     }
 
     /* construct the trace */
-    if (st->trace_level == LEARN_TRACER) {
+    if (st->trace_level == LEARN_TRACER && st->in_final_reduction_step == 0) {
         construct_trace(st->tr, mat);
     }
 
@@ -1423,7 +1445,7 @@ static void exact_sparse_reduced_echelon_form_ff_8(
 
     len_t npivs = 0; /* number of new pivots */
 
-    if (st->nf == 0) {
+    if (st->nf == 0 && st->in_final_reduction_step == 0) {
         dr      = realloc(dr, (unsigned long)ncols * sizeof(int64_t));
         mat->tr = realloc(mat->tr, (unsigned long)ncr * sizeof(hm_t *));
 
@@ -2094,9 +2116,9 @@ static void probabilistic_sparse_linear_algebra_ff_8(
     rt0 = realtime();
 
     /* allocate temporary storage space for sparse
-     * coefficients of new pivot rows */
+     * coefficients of all pivot rows */
     mat->cf_8 = realloc(mat->cf_8,
-            (unsigned long)mat->nrl * sizeof(cf8_t *));
+            (unsigned long)mat->nr * sizeof(cf8_t *));
     probabilistic_sparse_reduced_echelon_form_ff_8(mat, bs, st);
 
     /* timings */
@@ -2126,9 +2148,9 @@ static int exact_application_sparse_linear_algebra_ff_8(
     int ret;
 
     /* allocate temporary storage space for sparse
-     * coefficients of new pivot rows */
+     * coefficients of all pivot rows */
     mat->cf_8  = realloc(mat->cf_8,
-            (unsigned long)mat->nrl * sizeof(cf8_t *));
+            (unsigned long)mat->nr * sizeof(cf8_t *));
     ret = exact_application_sparse_reduced_echelon_form_ff_8(mat, bs, st);
 
     /* timings */
@@ -2159,9 +2181,9 @@ static void exact_trace_sparse_linear_algebra_ff_8(
     rt0 = realtime();
 
     /* allocate temporary storage space for sparse
-     * coefficients of new pivot rows */
+     * coefficients of all pivot rows */
     mat->cf_8  = realloc(mat->cf_8,
-            (unsigned long)mat->nrl * sizeof(cf8_t *));
+            (unsigned long)mat->nr * sizeof(cf8_t *));
     exact_trace_sparse_reduced_echelon_form_ff_8(trace, mat, bs, st);
 
     /* timings */
@@ -2190,9 +2212,9 @@ static void exact_sparse_linear_algebra_ff_8(
     rt0 = realtime();
 
     /* allocate temporary storage space for sparse
-     * coefficients of new pivot rows */
+     * coefficients of all pivot rows */
     mat->cf_8  = realloc(mat->cf_8,
-            (unsigned long)mat->nrl * sizeof(cf8_t *));
+            (unsigned long)mat->nr * sizeof(cf8_t *));
     exact_sparse_reduced_echelon_form_ff_8(mat, tbr, bs, st);
 
     /* timings */
